# -*- coding: utf-8 -*-

import os, sys, re, uuid, time, datetime, urllib, traceback
import xbmc, xbmcgui, xbmcaddon, xbmcplugin
import sdCommon, sdParser

from math import ceil
from hashlib import md5
from base64 import b64encode
from crypto.cipher.base import noPadding, BlockCipher
from crypto.cipher.aes_cbc  import AES_CBC

if sys.version_info >= (2,7): import json
else: import simplejson as json

ptv = xbmcaddon.Addon('plugin.video.polishtv.live')
sys.path.append(os.path.join(os.path.join(ptv.getAddonInfo('path'), '../resources'), 'lib'))

SERVICE_NAME = 'kinoman'
SERVICE_ICON = 'http://sd-xbmc.org/repository/xbmc-addons/'+ SERVICE_NAME + '.png'
COOKIES_FILE = os.path.sep.join([ ptv.getAddonInfo('path') , 'cookies', SERVICE_NAME + '.cookie' ])

API_USERNAME = ptv.getSetting('kinoman_login')
API_PASSWORD = ptv.getSetting('kinoman_password')

API_TEMP_DIR = xbmc.translatePath('special://temp') + SERVICE_NAME + os.path.sep
API_TEMP_WEB = 'cache' + os.path.sep
API_PAGE_URL = 'http://lajt.kinoman.tv/'
API_FILE_URL = 'http://static.kinoman.tv/'
API_MAIN_URL = 'http://api.lajt.kinoman.tv/api/'
API_DATA_URL = 'http://cache.lajt.kinoman.tv/api/'
API_DATA_KEY = 'function webpackJsonpCallback(chunkIds, moreModules)'
API_HOSTNAME = 'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/52.0.2743.116 Safari/537.36'

API_DEV_MODE = False
API_LOG_MODE = False
API_DEV_TEMP = False

def log(data):
	if API_LOG_MODE:
		file = open(API_TEMP_DIR + 'log.txt', 'a+')
		print >> file, '[%s] %s' % (datetime.datetime.now(), data)
		file.close()

class Kinoman:
	def __init__(self):
		self.__params = sdParser.Parser().getParams()

	def __getattr__(self, name):
		log('ATTR: ' + name)
		
		value = None
		
		if name == 'api':
			value = Kinoman_API()
		elif name == 'dialog':
			value = Kinoman_Dialog()
		elif name == '_reFormatNumber':
			value = re.compile('(\d)(?=(\d{3})+(?!\d))')
		elif name == '_reImagePath':
			value = re.compile('(\w+)(\w{2})(\w{2})')
		elif name == '_movieFanarts':
			value = self.api.getData('fanarts')
		
		if value is not None:
			setattr(self, name, value)
			return value
		
		raise AttributeError('%s instance has no attribute "%s"' % (self.__class__.__name__, name))

	def handleService(self):
		if self.getParam('cache') is None:
			Kinoman_API(True)
			self.forwardAction({ 'cache': '1' })
		else:
			method = self.getParam('action', 'forward' if self.getParam('forward') is not None else 'menu') + 'Action'
			log('---------------------------------')
			log('EXEC: %s (%s)' % (method, json.dumps(self.__params)))
			getattr(self, method)()

	def lock(self):
		xbmc.executebuiltin('ActivateWindow(busydialog)')

	def unlock(self):
		xbmc.executebuiltin('Dialog.Close(busydialog)')

	def getParam(self, name, default = None):
		try:
			return self.__params[name]
		except:
			return default

	def getUrl(self, link):
		data = { 'service': SERVICE_NAME, 'cache': '1' }
		if isinstance(link, basestring):
			link = { 'action': link }
		data.update(link)
		
		return sys.argv[0] + '?' + urllib.urlencode(data)

	def getImagePath(self, hash):
		return '/'.join(self._reImagePath.match(hash).groups())

	def getFolderName(self, name, size):
		if not isinstance(name, basestring):
			name = str(name)
		if len(name) > 0 and size is not None:
			name += ' [COLOR white]([B]%s[/B])[/COLOR]' % (self.formatNumber(size))
		return name

	def getVideoTitle(self, item):
		title = item['name'].encode('UTF-8')
		if item['orginal_name'] and len(item['orginal_name']):
			title += ' / ' + item['orginal_name'].encode('UTF-8')
		return title

	def getVideoIcon(self, item):
		return API_FILE_URL + 's/c/' + self.getImagePath(item['hash']) + '/o.jpg'

	def getActorIcon(self, item):
		if item['hasImage'] == '1':
			return API_FILE_URL + 's/o/' + item['personId'][:2] + '/' + item['personId'][2:][:2] + '/' + item['personId'] + '/o.jpg'
		return 'http://lajt.kinoman.tv//dist/0a6015b8908b2ab6215be82cc37a607a.png'

	def getMovieCode(self, item):
		label = item['type_short'].encode('UTF-8')
		color = { '_none': 'white', 'ENG': 'red', 'PL': 'lime', 'Lektor': 'green', 'Dubbing': 'blue', 'Napisy':  'yellow'}.get(label, '_none')
		return '[COLOR %s]%s[/COLOR]' % (color, label)

	def getMovieGenres(self, item):
		return ', '.join([ item['name'].encode('UTF-8') for item in self.api.getData('genres', 'id', item['genre_ids'].split(',')) ])

	def getMovieFanart(self, item):
		if int(item['id']) in self._movieFanarts:
			return API_FILE_URL + 's/s/' + self.getImagePath(item['hash']) + '/' + item['hash'] + '.jpg'

	def getSerieCode(self, item):
		year = []
		for key in [ 'startYear', 'endYear' ]:
			if item[key]:
				year.append(item[key])
		return '[COLOR white]%s[/COLOR]' % ('-'.join(year))

	def getEpisodeCode(self, item):
		return '[COLOR white]S[B]%02d[/B][/COLOR] [COLOR yellow]E[B]%02d[/B][/COLOR]' % (int(item['season']), int(item['episode']))

	def getLetters(self, numbers = False):
		letters = []
		if numbers:
			letters.append('0-9');
		for i in range(65, 91):
			letters.append(str(unichr(i)))
		return letters

	def formatNumber(self, number):
		return self._reFormatNumber.sub('\\1 ', str(number))

	def endDir(self, name = '_none'):
		data = {
			'_none':   { 'type': 'files',    'view': '50',          'mask': '%P' },
			'movie':   { 'type': 'movies',   'view': '504',     'mask': '%P [[COLOR white]%Y[/COLOR]] %R' },
			'serie':   { 'type': 'episodes',    'view': '504',     'mask': '%P %R' },
			'season':  { 'type': 'musicvideos',  'view': '504',     'mask': '%P' },
			'episode': { 'type': 'episodes', 'view': '504',     'mask': '%P' },
			'actor':   { 'type': 'artists',  'view': '500', 'mask': '%P' },
		}.get(name)
		
		if data:
			xbmc.executebuiltin('Skin.SetBool(AutoScroll)')
			xbmcplugin.addSortMethod(int(sys.argv[1]), sortMethod = xbmcplugin.SORT_METHOD_NONE, label2Mask = data['mask'])
			xbmcplugin.setContent(int(sys.argv[1]), data['type'])
			if 'confluence' in xbmc.getSkinDir():
				xbmc.executebuiltin('Container.SetViewMode(%s)' % (data['view']))
		xbmcplugin.endOfDirectory(int(sys.argv[1]))

	def addItem(self, item, link, isFolder):
		xbmcplugin.addDirectoryItem(int(sys.argv[1]), self.getUrl(link), item, isFolder)

	def addFolder(self, info, link):
		data = { 'thumb': SERVICE_ICON, 'size': None }
		
		if isinstance(info, basestring):
			info = { 'name': info }
		data.update(info)
		
		item = xbmcgui.ListItem(self.getFolderName(data['name'], data['size']))
		
		if data['thumb'] is not None:
			item.setArt({ 'thumb': data['thumb'] })
		
		self.addItem(item, link, True)

	def addAction(self, info, link, menu = []):
		data = { 'thumb': SERVICE_ICON, 'code': None }
		
		if isinstance(info, basestring):
			info = { 'title': info }
		data.update(info)
		
		item = xbmcgui.ListItem()
		
		item.addContextMenuItems(menu, True)
		
		item.setInfo('video', data)
		
		if data['thumb'] is not None:
			item.setArt({ 'thumb': data['thumb'] })
		
		self.addItem(item, link, False)

	def addVideo(self, info = {}, arts = {}, link = {}):
		item = xbmcgui.ListItem()
		item.setProperty('IsPlayable', 'true')
		item.setInfo('video', info)
		item.setArt(arts)
		self.addItem(item, { 'action': 'play', 'play': json.dumps(link) }, False)

	def addConfig(self, data):
		if isinstance(data, basestring):
			data = { 'config': data }
		
		config = self.api.getParamConfig(data['config'])
		if config is None:
			return
		
		if config['type'] == 'boolean':
			count = 1
			code = '[COLOR red]TAK[/COLOR]' if self.api.getParam(data['config']) else 'NIE'
		else:
			items = self.api.getData(config['data'], config['value'], self.api.getParam(data['config']))
			names = [ (str(item[config['short']]) if not isinstance(item[config['short']], basestring) else item[config['short']].encode('UTF-8')) for item in items ]
			count = len(items)
			color = 'yellow' if config['type'] == 'multiple' else 'lime'
			
			code = ', '.join([ '[COLOR %s]%s[/COLOR]' % (color, name) for name in names ]) if count else 'Wszystkie'
		
		link = { 'action': 'configDialog' }
		link.update(data)
		
		item = xbmcgui.ListItem()
		item.setArt({ 'thumb': SERVICE_ICON })
		item.setInfo('video', { 'title': self.getFolderName(config['title'], count if count > 1 else None), 'code': code })
		
		item.addContextMenuItems([('Resetuj filtr', 'RunPlugin(%s)' % (self.getUrl({ 'action': 'configClear', 'clear': data['config'], 'force': '1' })))], True)
		
		self.addItem(item, link, False)

	def addNext(self, query, count):
		offset = int(query['offset'])
		limit = int(query['limit'])
		page = int(offset / limit) + 1
		last = int(ceil(count / limit)) + 1
		next = offset + limit
		if (next < count):
			query['offset'] = next
			self.addFolder('[COLOR blue][ [B]%s[/B] z [B]%s[/B] ][/COLOR] [COLOR white]Następna strona >>>[/COLOR]' % (self.formatNumber(page), self.formatNumber(last)), { 'action': 'list', 'list': self.getParam('list'), 'query': json.dumps(query), 'page': next })

	def addMovie(self, item):
		self.addVideo({ 
			'title':  self.getVideoTitle(item),
			'code':   self.getMovieCode(item),
			'genre':  self.getMovieGenres(item),
			'plot':   '' if item['description'] is None else item['description'].encode('UTF-8'),
			'year':   item['year'],
			'rating': item['rate'],
			'votes':  item['rate_cnt']
		}, {
			'poster': self.getVideoIcon(item),
			'fanart': self.getMovieFanart(item) or self.getVideoIcon(item)
		}, { 
			'movie_id': item['id'] 
		})

	def addSerie(self, item):
		self.addAction({ 
			'title':     self.getVideoTitle(item),
			'code':      self.getSerieCode(item),
			'thumb':     self.getVideoIcon(item),
			'fanart':    self.getVideoIcon(item),
			'plot':      item['description'].encode('UTF-8'),
			'year':      item['startYear'],
			'premiered': str(item['updated']).split(' ')[0],
			'dateadded': item['updated'],
			'episode':   item['episode_cnt'],
			'rating':    item['rate'],
			'votes':     item['rate_cnt'],
		}, {
			'forward':   'menuSeriesSeasons',
			'thumb':     self.getVideoIcon(item),
			'serie':     item['id'],
			'count':     item['season_cnt'],
			'episode':   item['episode_cnt'],
			'query':     json.dumps({ 'series_id': item['id'] }),
		})

	def addEpisode(self, item):
		self.addVideo({ 
			'code':       self.getEpisodeCode(item),
			'title':      item['name'].encode('UTF-8'),
			'plot':       item['series_description'].encode('UTF-8'),
			'season':     item['season'],
			'episode':    item['episode'],
			'dateadded':  item['created'],
			'premiered':  str(item['updated']).split(' ')[0],
		}, {
			'poster':     self.getVideoIcon(item),
		}, {
			'episode_id': item['id'],
		})

	def addActor(self, item):
		self.addFolder({ 'name': item['name'].encode('UTF-8'), 'thumb': self.getActorIcon(item) }, { 'action': 'list', 'list': 'movie', 'query':  json.dumps({ 'actor_id': item['id'] })})

	def configDialogAction(self):
		name = self.getParam('config', '')
		
		config = self.api.getParamConfig(name)
		if config is None:
			return None
		
		dialog = getattr(self.dialog, { 'single': 'select', 'multiple': 'multiselect', 'boolean': 'yesno' }.get(config['type']))
		if dialog is None:
			return
		
		if config['type'] == 'boolean':
			value = 'true' if dialog('Filtr aktywny?', config['title']) else ''
		else:
			labels = []
			values = []
			for item in self.api.getData(config['data']):
				label = self.getFolderName(item[config['label']], item[config['count']] if config['count'] else None)
				value = item[config['value']]
				if len(label):
					labels.append(label)
					values.append(value)
			choice = dialog(config['title'], labels)
			if choice is None or choice < 0:
				return
			value = [ values[i] for i in choice ] if isinstance(choice, list) else values[choice]
		
		self.api.setParam(name, value)
		self.refreshAction()

	def configClearAction(self):
		if self.getParam('force') == '1' or self.dialog.yesno('Wyczyść filtry?', 'Filtry listy zostaną przywrócone do wartości domyślnych.'):
			self.api.unsetParams(self.getParam('clear', ''))
			self.refreshAction()

	def refreshAction(self):
		xbmc.executebuiltin('Container.Refresh')

	def forwardAction(self, params = {}):
		data = { 'action': self.getParam('forward', 'menu') }
		data.update(params)
		log('FORW: ' + json.dumps(data))
		xbmc.executebuiltin('Container.Update(%s%s&%s)' % (sys.argv[0], sys.argv[2], urllib.urlencode(data)))

	def findDialogAction(self):
		value = self.dialog.input(self.getParam('title', 'Wyszukaj'))
		if value:
			self.forwardAction({ 'query': json.dumps({ 'name_like': value }) })

	def menuAction(self):
		self.addFolder('Filmy',   'menuMovies')
		self.addFolder('Seriale', 'menuSeries')
		self.addFolder('Aktorzy', 'menuActors')
		self.endDir()

	def menuMoviesAction(self):
		query = {
			'genre':      self.api.getParam('movie_genres'),
			'type':       self.api.getParam('movie_types'),
			'year':       self.api.getParam('movie_years'),
			'rate':       self.api.getParam('movie_votes'),
			'quality':    self.api.getParam('movie_sizes'),
			'limit':      '1',
			'offset':     '0',
		}
		
		list = self.api.getList('movie', query)
		
		query.update({
			'order[col]': self.api.getParam('movie_sort'),
			'order[dir]': self.api.getParam('movie_order'),
			'limit':      self.api.getParam('movie_limit'),
		})
		
		self.addFolder({ 'name': 'Lista filmów', 'size':  list['cnt'] }, { 'action': 'list', 'list': 'movie', 'query': json.dumps(query) })
		self.addConfig('movie_sort')
		self.addConfig('movie_order')
		self.addConfig('movie_types')
		self.addConfig('movie_genres')
		self.addConfig('movie_years')
		self.addConfig('movie_votes')
		self.addConfig('movie_sizes')
		self.addAction('Wyszukaj', { 'action': 'findDialog', 'forward': 'list', 'list': 'movie' })
		self.addAction('Resetuj filtry', { 'action': 'configClear', 'clear': 'movie' })
		
		self.endDir()

	def menuSeriesAction(self):
		query = {
			'kids':   self.api.getParam('serie_kids'),
			'limit':  '1',
			'offset': '0',
		}
		
		list = self.api.getList('series', query)
		
		query.update({
			'order[col]': self.api.getParam('serie_sort'),
			'order[dir]': self.api.getParam('serie_order'),
			'limit':      self.api.getParam('serie_limit'), 
		})
		
		self.addFolder('Wszystkie seriale według litery', 'menuSeriesLetter')
		self.addFolder({ 'name': 'Lista seriali', 'size': list['cnt'] }, { 'action': 'list', 'list': 'serie', 'query': json.dumps(query) })
		self.addConfig('serie_sort')
		self.addConfig('serie_order')
		self.addConfig('serie_kids')
		self.addAction('Wyszukaj', { 'action': 'findDialog', 'forward': 'list', 'list': 'serie' })
		self.addAction('Resetuj filtry', { 'action': 'configClear', 'clear': 'serie' })
		self.endDir()

	def menuSeriesLetterAction(self):
		for item in self.api.getData('letters'):
			self.addFolder(item['letter'].encode('UTF-8'), { 'action': 'listSeriesLetter', 'letter': item['letter'].encode('UTF-8') })
		self.endDir()

	def menuSeriesSeasonsAction(self):
		try:
			serie = self.getParam('serie')
			start = int(self.api.getList('episodes', { 'series_id': serie })['rows'][0]['season'])
			count = int(self.getParam('count', '0'))
		except:
			return
		
		if serie is None:
			return
		
		thumb  = self.getParam('thumb')
		
		self.addFolder({ 
			'name':  'Wszystkie odcinki',
			'size':  self.getParam('episode', '0'),
			'thumb': thumb,
		}, { 
			'action':  'list',
			'list':    'episode',
			'query':   json.dumps({ 'series_id': serie })
		})
		
		for season in range(start, count + start):
			self.addAction({
				'title': 'Sezon ' + str(season),
				'thumb': thumb,
			}, {
				'forward': 'list',
				'list':    'episode',
				'query':   json.dumps({ 'series_id': serie, 'season': season })
			})
		
		self.endDir('season')

	def menuActorsAction(self):
		query = { 'limit': self.api.getParam('actor_limit'), 'offset': '0' }
		list = self.api.getList('actor', query)
		self.addFolder({ 'name': 'Lista wszystkich aktorów', 'size': list['cnt'] }, { 'action': 'list', 'list': 'actor', 'query': json.dumps(query) })
		self.addFolder('Lista według litery', 'menuActorsLetter')
		self.endDir()

	def menuActorsLetterAction(self):
		for letter in self.getLetters():
			self.addFolder(letter, { 'action': 'list', 'list': 'actor', 'query': json.dumps({ 'letter': letter }) })
		self.endDir()

	def listAction(self):
		name = self.getParam('list')
		
		data = {
			'movie':   { 'addMethod': 'addMovie',   'apiName': 'movie',    'listLimit': self.api.getParam('movie_limit')   },
			'serie':   { 'addMethod': 'addSerie',   'apiName': 'series',   'listLimit': self.api.getParam('serie_limit')   },
			'episode': { 'addMethod': 'addEpisode', 'apiName': 'episodes', 'listLimit': self.api.getParam('episode_limit') },
			'actor':   { 'addMethod': 'addActor',   'apiName': 'actor',    'listLimit': self.api.getParam('actor_limit')   },
		}.get(self.getParam('list'))
		
		if data is None:
			log('EXIT: List (%s)' % (name))
			return
		
		query = {
			'offset': '0',
			'limit': data['listLimit'],
		}
		query.update(json.loads(self.getParam('query', '{}')))
		
		push = getattr(self, data['addMethod'])
		list = self.api.getList(data['apiName'], query)
		for item in list['rows']: push(item)
		
		self.addNext(query, list['cnt'])
		self.endDir(name)

	def listSeriesLetterAction(self):
		letter = self.getParam('letter', None)
		if letter is None:
			return
		reLetter = re.compile('^\d' if letter == '0-9' else '^' + letter, re.I)
		list = self.api.getList('series')
		for item in list['rows']:
			if reLetter.match(item['name'].encode('UTF-8')):
				self.addSerie(item)
		self.endDir('serie')

	def playAction(self):
		play = json.loads(self.getParam('play', '{}'))
		if len(play) < 1:
			return
		log('PLAY: ' + json.dumps(play))
		link = self.api.getLink(play)
		log('PLAY: ' + link)
		item = xbmcgui.ListItem(path = link)
		xbmcplugin.setResolvedUrl(int(sys.argv[1]), True, item)

class Kinoman_API:

	def __init__(self, clean = False):
		self.common = sdCommon.common()
		
		if clean:
			self.__clean()
		
		self.__storagePassword = md5(str(uuid.uuid3(uuid.NAMESPACE_DNS, SERVICE_NAME))).hexdigest()
		
		self.__paramsConfig = {
			'movie_sort':   { 'data': 'sorts',   'type': 'single',   'label': 'name' , 'short': 'name',  'value': 'id', 'count': '',    'default': 'created', 'title': 'Sortowanie' },
			'movie_order':  { 'data': 'orders',  'type': 'single',   'label': 'name',  'short': 'name',  'value': 'id', 'count': '',    'default': 'desc',    'title': 'Kolejność' },
			'movie_genres': { 'data': 'genres',  'type': 'multiple', 'label': 'name',  'short': 'name',  'value': 'id', 'count': 'cnt', 'default': [],        'title': 'Gatunek' },
			'movie_years':  { 'data': 'years',   'type': 'multiple', 'label': 'label', 'short': 'label', 'value': 'id', 'count': '',    'default': [],        'title': 'Rok produkcji' },
			'movie_votes':  { 'data': 'votes',   'type': 'multiple', 'label': 'label', 'short': 'label', 'value': 'id', 'count': '',    'default': [],        'title': 'Ocena' },
			'movie_types':  { 'data': 'types',   'type': 'multiple', 'label': 'name',  'short': 'short', 'value': 'id', 'count': '',    'default': [],        'title': 'Rodzaj' },
			'movie_sizes':  { 'data': 'sizes',   'type': 'multiple', 'label': 'name',  'short': 'name',  'value': 'id', 'count': '',    'default': [],        'title': 'Jakość' },
			#'movie_limit':  { 'data': 'limits',  'type': 'single',   'label': 'name',  'short': 'name',  'value': 'id', 'count': '',    'default': '20',      'title': 'Limit strony' },
			'serie_sort':   { 'data': 'sorts2',  'type': 'single',   'label': 'name' , 'short': 'name',  'value': 'id', 'count': '',    'default': 'name',    'title': 'Sortowanie' },
			'serie_order':  { 'data': 'orders',  'type': 'single',   'label': 'name',  'short': 'name',  'value': 'id', 'count': '',    'default': 'asc',     'title': 'Kolejność' },
			'serie_kids':   { 'type': 'boolean', 'default': '', 'title': 'Dla dzieci' },
			'user_token':   { 'type': 'system', 'default': '' }
		}
		
		self.__dataConfig = {
			'genres':  {},
			'types':   {},
			'sizes':   {},
			'years':   {},
			'votes':   {},
			'sorts':   {},
			'sorts2':  {},
			'orders':  {},
			'letters': {},
			'fanarts': {},
		}
		
		self.__params = {
			'movie_limit':   '20',
			'actor_limit':   '28',
			'serie_limit':   '30',
			'episode_limit': '50',
		}
		
		self.__data = {
			'sizes': [
				{ 'id': '1', 'name': u'Niska' },
				{ 'id': '2', 'name': u'Średnia' },
				{ 'id': '3', 'name': u'Wysoka' },
			],
			'sorts2': [
				{ 'id': 'name',        'name': u'Tytuł' },
				{ 'id': 'rate',        'name': u'Ocena' },
				{ 'id': 'fav_cnt',     'name': u'Polubienia' },
				{ 'id': 'season_cnt',  'name': u'Liczba sezonów' },
				{ 'id': 'episode_cnt', 'name': u'Liczba epizodów' },
				{ 'id': 'year',        'name': u'Rok produkcji' },
				{ 'id': 'views',       'name': u'Wyświetlenia' },
				{ 'id': 'updated',     'name': u'Aktualizacja' },
				{ 'id': 'kids',        'name': u'Dla dzieci' },
			],
			'orders': [
				{ 'id': 'desc', 'name': u'Malejąco' },
				{ 'id': 'asc',  'name': u'Rosnąco' },
			],
		}
		
		self.__initData()
		self.__initParams()
		self.__initUser()

	def __clean(self):
		for name in [ API_TEMP_DIR, API_TEMP_DIR + API_TEMP_WEB ]:
			if not os.path.exists(name):
				os.makedirs(name)
			else:
				for item in [os.path.join(name, file) for file in os.listdir(name)]:
					if os.path.isfile(item):
						try:
							if API_DEV_TEMP:
								log('TEMP: ' + item)
							else:
								os.unlink(item)
						except: 
							pass
		log('=================================')

	def __encode(self, text):
		key = self.__storagePassword
		max = len(key)
		enc = ''
		for i in range(len(text)):
			key_c = key[i % max]
			enc_c = chr((ord(text[i]) + ord(key_c)) % 256)
			enc += enc_c
		return enc

	def __decode(self, data):
		key = self.__storagePassword
		max = len(key)
		dec = ''
		for i in range(len(data)):
			key_c = key[i % max]
			dec_c = chr((256 + ord(data[i]) - ord(key_c)) % 256)
			dec += dec_c
		return dec

	def __request(self, link, GET = None, POST = None, cache = True, cookie = False):
		request = { 'return_data': True }
		
		if POST:
			log('POST: %s' % (POST if isinstance(POST, basestring) else json.dumps(POST)))
			cache = False
			request.update({ 'use_post':  True })
			if isinstance(POST, basestring):
				request.update({ 'raw_post_data': True })
		
		if GET:
			link = self.__makeLink(link, GET)
		
		log('LINK: ' + link)
		
		if cache:
			hash = md5(link).hexdigest()
			file = API_TEMP_WEB + self.__fileName(hash)
			data = self.__fileLoad(file, None)
			if data is not None:
				return data
		
		try:
			request.update({ 'url':  link })
			if cookie:
				request.update({'use_cookie': True, 'save_cookie': True, 'load_cookie': os.path.isfile(COOKIES_FILE), 'cookiefile': COOKIES_FILE})
			data = self.common.getURLRequestData(request, POST)
		except Exception, exception:
			log('EXIT: ' + str(exception))
			traceback.print_exc()
			exit()
		
		if cache:
			self.__fileSave(file, data)
		
		return data

	def __apiLoad(self, link, data = None, default = None):
		token = self.getParam('user_token')
		if len(token):
			data.update({ 'token': token })
		result = self.__jsonLoad(self.__request(API_MAIN_URL + link, { 'apiData': self.__apiData(link), 'userToken': token }, json.dumps(data) ))
		
		if isinstance(default, dict):
			default.update(result)
			return default
		
		if isinstance(default, list):
			return default + result
		
		return result

	def __apiData(self, data):
		klen = 32
		ilen = 16
		salt = os.urandom(8)
		
		plaintext = json.dumps(data)
		password = API_DATA_KEY.encode('ascii','ignore')
		
		try:
			maxlen = klen + ilen
			keyiv = md5(password + salt).digest()
			tmp = [keyiv]
			while len(tmp) < maxlen:
				tmp.append( md5(tmp[-1] + password + salt).digest() )
				keyiv += tmp[-1]
				key = keyiv[:klen]
				iv = keyiv[klen:klen + ilen]
		except UnicodeDecodeError:
			return ''
		
		padding_len = 16 - (len(plaintext) % 16)
		padded_plaintext = plaintext + (chr(padding_len) * padding_len)
		
		cipher = AES_CBC(key, noPadding(), klen)
		ciphertext = cipher.encrypt(padded_plaintext, iv)
		
		result = json.dumps({ 'ct': b64encode(ciphertext), 'iv': iv.encode('hex'), 's': salt.encode('hex') })
		
		log('DATA: %s = %s' % (plaintext, result))
		
		return result

	def __makeLink(self, link, data):
		query = []
		for name, value in data.items():
			if isinstance(value, int) or len(value):
				if isinstance(value, list):
					for item in value:
						query.append('%s[]=%s' % (name, item))
				else:
					query.append(urllib.urlencode({ name: value }))
		if (len(query)):
			query.sort()
			link += ('&' if link.find('?') > -1 else '?') + '&'.join(query)
		return link

	def __initParams(self):
		data = self.__loadParams()
		for name, value in data.items():
			self.setParam(name, value, False)

	def __initData(self):
		data = self.__loadData()
		for name, config in self.__dataConfig.items():
			try:
				if len(data[name]) > 0:
					self.__data[name] = data[name]
			except:
				pass

	def __initUser(self):
		token = self.getParam('user_token')
		if len(token) > 0:
			return
		
		if len(API_USERNAME) < 1 or len(API_PASSWORD) < 1:
			return
		
		if self.__userLogin():
			log('USER: ' + self.getParam('user_token'))
			user = self.getUser()
			head = 'Zalogowano: ' + user['username']
			if user['premium_valid']:
				text = 'PREMIUM: ' + user['premium_valid']
				icon = xbmcgui.NOTIFICATION_INFO
			else:
				text = 'Konto STANDARD'
				icon = xbmcgui.NOTIFICATION_WARNING
		else:
			log('USER: Invalid')
			head = 'Błąd logowania'
			text = 'Nieprawidłowy login lub hasło'
			icon = xbmcgui.NOTIFICATION_ERROR
		
		xbmcgui.Dialog().notification(head, text, icon)

	def __fileName(self, name):
		if API_DEV_MODE:
			return name + '.txt'
		return md5(name).hexdigest()

	def __fileSave(self, name, content = '', crypt = False):
		if API_DEV_MODE:
			crypt = False
		try:
			file = open(API_TEMP_DIR + name, 'w')
			file.write(content if crypt == False else self.__encode(content))
			file.close()
			log('SAVE: ' + name)
		except:
			return False
		return True

	def __fileLoad(self, name, default = '', crypt = False):
		if API_DEV_MODE:
			crypt = False
		try:
			file = open(API_TEMP_DIR + name, 'r')
			read = file.read()
			file.close()
			log('LOAD: ' + name)
		except:
			return default
		return read if crypt == False else self.__decode(read)

	def __saveParams(self):
		self.__fileSave(self.__fileName('params'), self.__jsonDump(self.__params), True)

	def __loadParams(self):
		return self.__jsonLoad(self.__fileLoad(self.__fileName('params'), '{}', True))

	def __saveData(self):
		self.__fileSave(self.__fileName('data'), self.__jsonDump(self.__data), True)

	def __loadData(self):
		return self.__jsonLoad(self.__fileLoad(self.__fileName('data'), '{}', True))

	def __loadDataBundle(self):
		log('INIT: Bundle')
		bundle = self.__request(API_PAGE_URL + 'dist/app.bundle.js')
		for name in [ 'years', 'votes', 'sorts' ]:
			if name == 'sorts':
				sorts = re.finditer('<li[^>]+ng-click="sort\(\\\\\'(?P<id>\w+)\\\\\'\)"[^>]*>(?P<name>[^<]+)</li>', bundle, re.I + re.M + re.S)
				value = [ { 'id': 'created', 'name': u'Data dodania' } ] + [ sort.groupdict() for sort in sorts ]
			else:
				match = re.search(name + '=(\[[^\]]+\])', bundle)
				value = self.__jsonLoad(match.groups()[0], True) if match else self.__data[name]
			self.__data[name] = value

	def __loadDataFilters(self):
		log('INIT: Filters')
		filter = json.loads(self.__request(API_DATA_URL + 'movie_filter'))
		for name in [ 'types', 'genres' ]:
			self.__data[name] = filter[name]

	def __loadDataLetters(self):
		log('INIT: Letters')
		isnum = re.compile('\d')
		letters = []
		cnt = 0
		cur = ''
		prv = '0-9'
		for item in self.getList('series')['rows']:
			cur = item['name'][:1]
			if isnum.match(cur):
				cur = prv
			if cur != prv:
				letters.append({ 'letter': prv, 'count': cnt })
				cnt = 0
			cnt += 1
			prv = cur
		if cnt > 0:
			letters.append({ 'letter': cur, 'count': cnt })
		self.__data['letters'] = letters

	def __loadDataFanarts(self):
		log('INIT: Fanarts')
		fanarts = []
		list = self.getList('movie', { 'hasSlideshow': '1', 'offset': '0', 'limit': '100' })
		for item in list['rows']:
			fanarts.append(int(item['id']))
		self.__data['fanarts'] = fanarts

	def __jsonLoad(self, text, fix = False):
		try:
			data = json.loads(re.sub(r'([\w_]+)(?=:)', r'"\1"', text) if fix else text)
		except:
			data = None
		return data if data else {}

	def __jsonDump(self, data):
		try:
			text = json.dumps(data, indent = 4, separators = (',', ': ')) if API_DEV_MODE else json.dumps(data)
		except:
			return '{}'
		return text

	def __userLogin(self):
		log('USER: Login')
		user = self.__apiLoad('user/login', { 'username': API_USERNAME, 'password': API_PASSWORD }, { 'token': '0' })
		self.setParam('user_token', user['token'])
		return len(user['token']) > 1

	def __videoLink(self, code):
		link = ''
		
		if self.getUser()['premium_valid']:
			hash = self.__apiLoad('player/get', { 'action': 'getHash', 'code': code })
			log('HASH: ' + hash)
			link = self.__apiLoad('player/get', { 'action': 'getPlayerByHash', 'hash': hash })
		else:
			expr = re.compile('\s+', re.M)
			free = self.__apiLoad('player/get', { 'action': 'getFreePlayer', 'code': code }).encode('UTF-8')
			free = expr.sub(' ', free)
			log('FREE: ' + free)
			page = re.search('src="([^"\?]+)', free).groups()[0]
			
			html = self.__request(page, cache = False, cookie = True)
			html = expr.sub(' ', html)
			log('HTML: ' + html)
			
			dialog = Kinoman_Dialog()
			#regexp = re.compile('<img[^>]+src="(/simple-php-captcha-master[^"]+)"[^>]*>')
			regexp = re.compile('"(/simple-php-captcha-master[^"]+)"')
			
			while True:
				captcha = regexp.search(html)
				if captcha:
					img = 'http://www.vidzer.net' + captcha.groups()[0].replace('&amp;', '&')
					log('CAPT: ' + img)
					
					file = API_TEMP_DIR + API_TEMP_WEB + md5(img).hexdigest() + '.png'
					
					f = open(file, 'wb')
					f.write(self.__request(img, cache = False, cookie = True))
					f.close()
					
					text = dialog.captcha(file)
					
					if len(text):
						html = self.__request(page, POST = { 'captcha': text }, cookie = True)
						html = expr.sub(' ', html)
					else:
						break
				else:
					break
			
			find = re.search('<a[^>]+href="([^"]+)"[^>]*id="player"[^>]*>', html)
			if find:
				link = find.groups()[0]
		
		return link

	def getUser(self):
		user = { 'id': '', 'username': '', 'premium_valid': '' }
		token = self.getParam('user_token')
		if len(token) > 1:
			return self.__apiLoad('user/getUser', { 'token': token }, user)
		return user

	def getLink(self, params):
		code = self.__apiLoad('link', params, [])
		log('CODE: ' + json.dumps(code))
		if len(code) > 0:
			return self.__videoLink(code[0]['code'])

	def getList(self, list, data = {}):
		result = { 'cnt': 0, 'rows': [] }
		result.update(self.__jsonLoad(self.__request(API_DATA_URL + list, data)))
		return result

	def getData(self, name, key = None, value = None):
		try:
			 self.__dataConfig[name]
		except:
			return []
		
		try:
			data = self.__data[name]
		except:
			if name in [ 'years', 'votes', 'sorts' ]:
				self.__loadDataBundle()
			elif name in [ 'types', 'genres' ]:
				self.__loadDataFilters()
			elif name in [ 'letters' ]:
				self.__loadDataLetters()
			elif name in [ 'fanarts' ]:
				self.__loadDataFanarts()
			else:
				log('DATA: Not found method for "%s"' %(name))
				return data
			
			try:
				data = self.__data[name]
			except:
				log('DATA: Error loading data for "%s"' %(name))
				return data
			
			self.__saveData()
		
		if key is None:
			return data
		
		find = []
		if not value is None and not isinstance(value, list):
			value = [ value ]
		for item in data:
			try:
				item[key]
				if value is None or item[key] in value:
					find.append(item)
			except:
				pass
		
		return find

	def getParamConfig(self, name):
		try:
			return self.__paramsConfig[name]
		except:
			return None

	def getParamOptions(self, name):
		try:
			return self.__paramsConfig[name]
		except:
			return None

	def unsetParams(self, starts = ''):
		for name, config in self.__paramsConfig.items():
			if name.startswith(starts):
				self.__params.pop(name, None)
		self.__saveParams()

	def setParam(self, name, value, save = True):
		config = self.getParamConfig(name)
		if config is None:
			return False
		
		if config['type'] == 'miltiple' and not isinstance(value, list):
			value = [ value ]
		
		self.__params[name] = value
		
		if save:
			self.__saveParams()

	def getParam(self, name):
		try:  
			return self.__params[name]
		except:
			config = self.getParamConfig(name)
			if config:
				return config['default']
		return None

class Kinoman_Dialog(xbmcgui.Dialog):
	def multiselect(self, heading, options):
		dialog = Kinoman_Dialog_Multiselect('DialogSelect.xml', '')
		dialog.setHeading(heading)
		dialog.setOptions(options)
		dialog.doModal()
		values = dialog.values
		del dialog
		return values

	def captcha(self, captcha):
		return Kinoman_Dialog_Captcha(captcha = captcha).getText()

class Kinoman_Dialog_Multiselect(xbmcgui.WindowXMLDialog):
	def setHeading(self, heading):
		self.__heading = heading
	
	def setOptions(self, options):
		self.__options = options
	
	def onInit(self):
		self.__temp = []
		self.values = None
		
		self.getControl(1).setLabel(self.__heading)
		self.getControl(3).addItems(self.__options)
		self.getControl(5).setLabel('OK')
		
		self.removeControl(self.getControl(6))
		self.getControl(3).setEnabled(True)
		self.setFocus(self.getControl(3))
	
	def onAction(self, action):
		actionId = action.getId()
		if actionId in [ 10, 12, 92 ]:
			self.close()
		elif self.getFocusId() == 3:
			if actionId in [ 7, 12, 100 ]:
				list = self.getFocus()
				item = list.getSelectedItem()
				
				index = list.getSelectedPosition()
				if index in self.__temp:
					item.select(False)
					self.__temp.remove(index)
				else:
					item.select(True)
					self.__temp.append(index)
	
	def onClick(self, controlId):
		if controlId == 5:
			self.values = self.__temp
			self.values.sort()
		self.close()

class Kinoman_Dialog_Captcha(xbmcgui.WindowDialog):
	def __init__(self, *args, **kwargs):
		self.addControl(xbmcgui.ControlImage(550, 30, 160, 75, kwargs.get('captcha')))
		self.keyboard = xbmc.Keyboard()

	def getText(self):
		self.show()
		self.keyboard.doModal()
		if (self.keyboard.isConfirmed()):
			text = self.keyboard.getText()
			self.close()
			return text
		self.close()
		return ''
